/******************************************************************************
* C Source: Astronomy Tools                                                   
*                                                                             
* File:                                                                       
*    astrmcal.c                                                               
*                                                                             
* Description:                                                                
*    This set of functions performs various astronomically related tasks.     
*    Unless, otherwise noted, these functions were either originally written, 
*    or ported to C by R. White.                                              
*                                                                             
* Module List:                                                                
*    b1950_j2000                                                              
*       Converts equatorial coordinates from B1950 to J2000 equinoxes         
*       assuming that proper motion and parallax in J2000 frame are zero.     
*    condmsrd                                                                 
*       Converts equatorial coordinates in hours, minutes, seconds, degrees   
*       arcminutes, arcseconds to radians, radians.                           
*    condrddms                                                                
*       Converts equatorial coordinates in radians, radians to hours, minutes 
*       seconds, degrees.                                                     
*    Great_Circle_Separation                                                  
*       Computes the great circle distance between to equatorial coordinate   
*       positions.                                                            
*    Read_DMS                                                                 
*       Reads from a file, equatorial coordinates in the format of hours,     
*       minutes, seconds, degrees, arcminutes, arcseconds into radians.       
*    traneqstd                                                                
*       Converts equatorial coordinates to standard coordinates for a plate.  
*    transtdeqs                                                               
*       Converts standard coordinates to equatorial coordinates for a plate.  
*    Write_DMS                                                                
*       Writes to a file, equatorial coordinates given in radians, in the     
*       format of hours, minutes, seconds, degrees, arcminutes, arcseconds.   
*                                                                             
* History:                                                                    
*    Created    : 13-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 26-MAY-94 by J. Doggett,                                    
*       Updated Write_DMS function.                                           
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 18-DEC-96 by J. Doggett,                                    
*       Redelivered condmsrd and Read_DMS functions.
*    Updated    : 19-DEC-96 by J. Doggett,
*       Redelivered Write_DMS function.
*    Redelivered: xxx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.   
*    Updated	: 01-Aug-2005 B.McLean
*		Spherical projection for GSC-II
*		GetImage v3.x
*                                                                             
* Copyright (c) 1993, 1994, Association of Universities for Research in       
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <ctype.h>
#include <math.h>
#include <stdio.h>

#include "astronmy.h"

#include "generic.h"

/******************************************************************************
* Module:                                                                     
*    b1950_j2000                                                              
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    Convert from B1950 coordinates to J2000.                                 
*                                                                             
*    Conversion between B1950 and J2000 coordinates taken from Aoki, X.,      
*    Soma, M., Kinoshita, H., Inoue, K. (1983) A.Ap. 128, 263. and            
*    Astronomical Almanac, 1989, pp. B42-B43 and simplified for the case of   
*    no proper motions or parallaxes in J2000 coordinate system. Note that a  
*    fixed target would have a spurious proper motion (the E-term of the      
*    aberration) in the B1950 coordinates, so for fixed targets the correct   
*    assumption is no PM in the J2000 frame.                                  
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       in_ra                                                                 
*          double, Value                                                      
*             B1950 right ascension to convert to J2000.                      
*       in_dec                                                                
*          double, Value                                                      
*             B1950 declination to convert to J2000.                          
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       out_ra                                                                
*          double, Value                                                      
*             J2000 right ascension converted from B1950.                     
*       out_dec                                                               
*          double, Value                                                      
*             J2000 declination converted from B1950.                         
*                                                                             
* History:                                                                    
*    Created    : 05-APR-89 by R. White,                                      
*    Delivered  : 05-APR-89 by R. White,                                      
*                                                                             
*    Updated    : 15-JUN-93 by J. Doggett,                                    
*       Fixed original failure to adjust right ascension when it falls in the 
*          range -pi/2 to 0.                                                  
*    Updated    : 13-AUG-93 by J. Doggett,                                    
*       Adopted into software used to facilitate the accessing of data from   
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
static double a   [3]    = { -1.62557e-6, -0.31919e-6, -0.13843e-6 };
static double adot[3]    = {  1.244e-3  , -1.57900e-3, -0.66000e-3 };
static double m   [6][6] = {
    {  0.9999256782    , -0.0111820611    , -0.0048579477    ,
       0.00000242395018, -0.00000002710663, -0.00000001177656  },
    {  0.0111820610    ,  0.9999374784    , -0.0000271765    ,
       0.00000002710663,  0.00000242397878, -0.00000000006587  },
    {  0.0048579479    , -0.0000271474    ,  0.9999881997    ,
       0.00000001177656, -0.00000000006582,  0.00000242410173  },
    { -0.000551        , -0.238565        ,  0.435739        ,
       0.99994704      , -0.01118251      , -0.00485767         },

/* *** b1950_j2000 ***                                                     2 */


    {  0.238514        , -0.002667        , -0.008541        ,
       0.01118251      ,  0.99995883      , -0.00002718         },
    { -0.435623        ,  0.012254        ,  0.002117        ,
       0.00485767      , -0.00002714      ,  1.00000956         } };

extern void
b1950_j2000(in_ra,in_dec,out_ra,out_dec)
double in_ra,in_dec;
double *out_ra,*out_dec;
{
double r0[3], r[6], a1[3];
double ar = 0.0, cd, t1, t;
int i, j;

    cd = cos(in_dec);
    r0[0] = cd*cos(in_ra);
    r0[1] = cd*sin(in_ra);
    r0[2] = sin(in_dec);

    /* correct for E-terms of aberration
     * assume epoch of observation was about 1980; there ought to be
     * another version of the program when the epoch of the observations
     * are known.
     */
    t1 = (1980.0 - 1950.0)/100.0;
    t = (t1-0.5)*(PI/(180.0*3600.0));
    t1 = t1*(PI/(180.0*3600.0));
    for (i=0; i<3; i++) a1[i] = a[i] + adot[i]*t1;
    for (i=0; i<3; i++) ar += r0[i]*a1[i];
    for (i=0; i<3; i++) r0[i] = (1.0+ar)*r0[i] - a1[i];

    /* precess from 1950 to 2000 */
    for(i=0; i<6; i++) {
        r[i] = 0.0;
        for (j=0; j<3; j++) r[i] += m[i][j]*r0[j];
    }
    /* modify r for assumed 1980 epoch */
    for (i=0; i<3; i++) r[i] += r[i+3]*t;

    /* convert from x,y,z to RA, Dec */
    *out_dec = asin( r[2] / sqrt(r[0]*r[0]+r[1]*r[1]+r[2]*r[2]) );
    *out_ra = atan2(r[1],r[0]);

/* Adjust out_ra if it falls in range -pi/2 to 0. */
    if ( *out_ra < 0. )
       *out_ra += 2.*PI;

} /* b1950_j2000 */

/******************************************************************************
* Module:                                                                     
*    condmsrd                                                                 
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    This routine takes the Right Ascension and Declination and converts from 
*    traditional units to radians.                                            
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       ra_h                                                                  
*          int, Value                                                         
*             Input right ascension hours.                                    
*       ra_m                                                                  
*          int, Value                                                         
*             Input right ascension minutes.                                  
*       ra_s                                                                  
*          double, Value                                                      
*             Input right ascension seconds.                                  
*       dec_sign                                                              
*          char, Value                                                        
*             Input declination sign.                                         
*       dec_d                                                                 
*          int, Value                                                         
*             Input declination degrees.                                      
*       dec_m                                                                 
*          int, Value                                                         
*             Input declination arcminutes.                                   
*       dec_s                                                                 
*          double, Value                                                      
*             Input declination arcseconds.                                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       object_ra                                                             
*          double, Reference                                                  
*             Output right ascension in radians.                              
*       object_dec                                                            
*          double, Reference                                                  
*             Output declination in radians.                                  
*                                                                             
* History:                                                                    
*    Created    : 04-MAY-90 by R. White,                                      
*       Converted from IDL.                                                   
*    Delivered  : 04-MAY-90 by R. White,                                      
*                                                                             
*    Updated    : 13-AUG-93 by J. Doggett,                                    
*       Adopted into software used to facilitate the accessing of data from   
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*
*    Updated    : 18-DEC-96 by J. Doggett,
*       Modified to normalize RA to range from 0 <= RA < 2*PI.
*    Redelivered: 18-DEC-96 by J. Doggett,
*       GetImage Version 2.0x.
******************************************************************************/

/* *** condmsrd ***                                                        2 */


extern void
condmsrd(ra_h, ra_m, ra_s, dec_sign, dec_d, dec_m, dec_s,
    object_ra, object_dec)
int ra_h, ra_m;
double ra_s;
char dec_sign;
int dec_d, dec_m;
double dec_s;
double *object_ra, *object_dec;
{
    /*
     *  Convert right ascension
     */
    *object_ra = (ra_h*3600.0+ra_m*60.0+ra_s)/SECONDS_PER_RADIAN;

     while (*object_ra >= 2*PI)
        *object_ra -= 2*PI;

     while (*object_ra < 0)
        *object_ra += 2*PI;

    /*
     *  Convert declination
     */
    *object_dec = (dec_d*3600.0+dec_m*60.0+dec_s)/ARCSECONDS_PER_RADIAN;
    if (dec_sign == '-') *object_dec = -(*object_dec);

} /* condmsrd */

/******************************************************************************
* Module:                                                                     
*    conrddms                                                                 
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    This routine takes the Right Ascension and Declination and converts from 
*    radians to traditional units.                                            
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       ra                                                                    
*          double, Value                                                      
*             Input right ascension in radians.                               
*       dec                                                                   
*          double, Value                                                      
*             Input declination in radians.                                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       ra_h                                                                  
*          int, Reference                                                     
*             Output right ascension hours.                                   
*       ra_m                                                                  
*          int, Reference                                                     
*             Output right ascension minutes.                                 
*       ra_s                                                                  
*          double, Reference                                                  
*             Output right ascension seconds.                                 
*       dec_sign                                                              
*          char, Reference                                                    
*             Output declination sign.                                        
*       dec_d                                                                 
*          int, Reference                                                     
*             Output declination degrees.                                     
*       dec_m                                                                 
*          int, Reference                                                     
*             Output declination arcminutes.                                  
*       dec_s                                                                 
*          double, Reference                                                  
*             Output declination arcseconds.                                  
*                                                                             
* History:                                                                    
*    Created    : 31-JUL-91 by R. White,                                      
*       Converted from IDL to C.                                              
*    Delivered  : 31-JUL-91 by R. White,                                      
*                                                                             
*    Updated    : 13-AUG-93 by J. Doggett,                                    
*       Adopted into software used to facilitate the accessing of data from   
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
extern void
conrddms(ra,dec,ra_h,ra_m,ra_s,dec_sign,dec_d,dec_m,dec_s)
double ra, dec;
int *ra_h, *ra_m;
double *ra_s;

/* *** conrddms ***                                                        2 */


char *dec_sign;
int *dec_d, *dec_m;
double *dec_s;
{
double object_ra, object_dec;

    /*
     * Convert right ascension
     * Assume 0 <= ra < 2*pi
     */
    object_ra = ra*SECONDS_PER_RADIAN;
    *ra_h = (int) (object_ra/3600);
    *ra_m = (int) ((object_ra- (*ra_h)*3600)/60);
    *ra_s = object_ra - ((*ra_h)*60 + (*ra_m))*60;
    /*
     * Convert declination
     */
    if (dec < 0) {
        *dec_sign = '-';
    } else {
        *dec_sign = '+';
    }
    object_dec = fabs(dec)*ARCSECONDS_PER_RADIAN;

    *dec_d = (int) (object_dec/3600);
    *dec_m = (int) ((object_dec - (*dec_d)*3600)/60);
    *dec_s = object_dec - ((*dec_d)*60 + (*dec_m))*60;
} /* conrddms */

/******************************************************************************
* Module:                                                                     
*    Great_Circle_Separation                                                  
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    This routine computes the great circle distance between two sets of      
*    equatorial coordinates.                                                  
*                                                                             
* Return Value:                                                               
*    double: The great circle distance in radians.                            
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       ra_1                                                                  
*          double, Value                                                      
*             Right ascension of one set of equatorial coordinates.           
*       dec_1                                                                 
*          double, Value                                                      
*             Declination of one set of equatorial coordinates.               
*       ra_2                                                                  
*          double, Value                                                      
*             Right ascension of another set of equatorial coordinates.       
*       dec_2                                                                 
*          double, Value                                                      
*             Declination of another set of equatorial coordinates.           
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       Adopted from GASP Fortran into C for software used to facilitate the  
*          accessing of data from the CD-ROM set of the Digitized Sky Survey. 
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/***************************************
* Great_Circle_Separation declarations *
***************************************/
double Great_Circle_Separation( RA_1, Dec_1, RA_2, Dec_2 )
double RA_1 ;
double Dec_1;
double RA_2 ;
double Dec_2;
{
   double Separation; /* Separation result */
   double Epsilon = GREAT_CIRCLE_EPSILON/ARCSECONDS_PER_RADIAN;
                      /* If right ascension and declination differences are *
                       * less then this, then zero is returned.             */

/************************************************
* Great_Circle_Separation begin executable code *
************************************************/
   if ( ( fabs( RA_1  - RA_2  ) < Epsilon ) &&
        ( fabs( Dec_1 - Dec_2 ) < Epsilon ) )
      Separation=0;
   else

      Separation= acos(

/* *** Great_Circle_Separation ***                                         2 */


            cos( PI/2. - Dec_2 )*cos( PI/2. - Dec_1 ) +
            sin( PI/2. - Dec_2 )*sin( PI/2. - Dec_1 )*cos( RA_1 - RA_2 ) );

   return Separation;
}
/***********************************
* Great_Circle_Separation end code *
***********************************/

/******************************************************************************
* Module:                                                                     
*    Read_DMS                                                                 
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    This routines reads equatorial coordinates in hours, minutes, seconds,   
*    seconds, degrees, arcminutes, arcseconds from a file into radians. This  
*    function handles the sign of the declination as optional.                
*                                                                             
* Return Value:                                                               
*    int: Normally, 1. If end-of-file encountered, then EOF. If other error   
*         reading, then NULL;                                                 
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Input_File                                                            
*          File*, Value                                                       
*             Pointer to file where coordinates are to be written.            
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       RA                                                                    
*          double, Reference                                                  
*             Right ascension to write.                                       
*       Dec                                                                   
*          double, Reference                                                  
*             Declination to write.                                           
*                                                                             
* History:                                                                    
*    Created    : 25-OCT-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*
*    Updated    : 18-DEC-96 by J. Doggett,
*       Modified to allow 24 hours, 60 minutes, seconds, arcminutes and
*          arcseconds.
*       Modified to reject declinations outside of +-90 degrees.
*    Redelivered: 18-DEC-96 by J. Doggett,
*       GetImage Version 2.0x.
******************************************************************************/
/************************
* Read_DMS declarations *
************************/
int     Read_DMS( Input_File, RA, Dec )
FILE   *Input_File;
double *RA        ;
double *Dec       ;
{
   int    RA_Hours      ; /* Place to hold input coordinates while they're */
   int    RA_Minutes    ; /* converted to radians.                         */
   double RA_Seconds    ;
   char   Dec_Sign      ;
   int    Dec_Degrees   ;
   int    Dec_ArcMinutes;
   double Dec_ArcSeconds;

/* *** Read_DMS ***                                                        2 */


   char Temp_Sign;

/*********************************
* Read_DMS begin executable code *
*********************************/
/* Read through possible declination sign. */
   switch ( fscanf( Input_File     ,
                    "%d %d %lf %1c",
                    &RA_Hours       ,
                    &RA_Minutes     ,
                    &RA_Seconds     ,
                    &Dec_Sign
                  )
           )
   {
      case 4:
         if ( ( Dec_Sign != '+' ) && ( Dec_Sign != '-' ) )
         {
            Temp_Sign = Dec_Sign;
            Dec_Sign  = '+';
            if (!isspace(Temp_Sign))
               ungetc( ( int )Temp_Sign, Input_File );
         }
         break;
      case EOF:
         return EOF;
      default:
         return (int)NULL;
   }

/* Check for valid right ascension */
   if ((RA_Hours   > 24) || (RA_Hours   < 0) ||
       (RA_Minutes > 60) || (RA_Minutes < 0) ||
       (RA_Seconds > 60) || (RA_Seconds < 0)
      )
      return (int)NULL;

/* Read past declination sign. */
   switch( fscanf( Input_File ,
                   "%d %d %lf",
                   &Dec_Degrees,
                   &Dec_ArcMinutes,
                   &Dec_ArcSeconds
                 )
          )
   {
      case 3:
		 condmsrd(RA_Hours, RA_Minutes, RA_Seconds, Dec_Sign, Dec_Degrees,
                  Dec_ArcMinutes, Dec_ArcSeconds, RA, Dec);
         break;
      case EOF:
         return EOF;
      default:
         return (int)NULL;
   }

/* Check for valid declination */
   if ((Dec_Degrees    > 90) || (Dec_Degrees    < -90) ||
       (Dec_ArcMinutes > 60) || (Dec_ArcMinutes <   0) ||
       (Dec_ArcSeconds > 60) || (Dec_ArcSeconds <   0) ||
       (fabs(*Dec) > ((double)90.001/180)*PI)

/* *** Read_DMS ***                                                        3 */

      )
      return (int)NULL;

   if (*Dec > ((double)90/180)*PI)
      *Dec =  ((double)90/180)*PI;

   if (*Dec < -((double)90/180)*PI)
      *Dec = -((double)90/180)*PI;

   return 1;
}
/*******************************
* Read_DMS end executable code *
*******************************/

/******************************************************************************
* Module:                                                                     
*    traneqstd                                                                
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    Routine to convert RA and Dec in radians to standard coordinates on a    
*    plate.                                                                   
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       plt_center_ra                                                         
*          double, Value                                                      
*             Plate center right ascension in radians                         
*       plt_center_dec                                                        
*          double, Value                                                      
*             Plate center declination in radians                             
*       object_ra                                                             
*          double, Value                                                      
*             Input right ascension in radians.                               
*       object_dec                                                            
*          double, Value                                                      
*             Input declination in radians.                                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       Object_Xi                                                             
*          double, Reference                                                  
*             Output xi standard coordinate in arcseconds.                    
*       Object_Eta                                                            
*          double, Reference                                                  
*             Output eta standard coordinate in arcseconds.                   
*                                                                             
* History:                                                                    
*    Created    : 07-MAY-90 by R. White,                                      
*       Converted to Fortran from IDL routine.                                
*    Delivered  : 30-JUL-91 by R. White,                                      
*       Convert from Fortran to C.                                            
*                                                                             
*    Updated    : 13-AUG-93 by J. Doggett,                                    
*       Adopted into software used to facilitate the accessing of data from   
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Updated    : 14-DEC-93 by J. Doggett,                                    
*       Modified to take plate center coordinates explicitly as arguments     
*          rather than from the plate header.                                 
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
extern void traneqstd(plt_center_ra, plt_center_dec,object_ra,object_dec,object_xi,object_eta)
double plt_center_ra, plt_center_dec;
double object_ra, object_dec;
double *object_xi, *object_eta;
{
double div;



/* *** traneqstd ***                                                       2 */


    /*
     *  Find divisor
     */

    div=(sin(object_dec)*sin(plt_center_dec)+
        cos(object_dec)*cos(plt_center_dec)*
        cos(object_ra - plt_center_ra));
    /*
     *  Compute standard coords and convert to arcsec
     */
    *object_xi=cos(object_dec)*sin(object_ra - plt_center_ra)*
        ARCSECONDS_PER_RADIAN/div;

    *object_eta=(sin(object_dec)*cos(plt_center_dec)-
        cos(object_dec)*sin(plt_center_dec)*
        cos(object_ra - plt_center_ra))*
        ARCSECONDS_PER_RADIAN/div;

}
/******************************************************************************
*    transtdeq                                                                
* Module:                                                                     
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*     Routine to convert standard coordinates on a plate to RA and Dec in     
*     radians.                                                                
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       plt_center_ra                                                         
*          double, Value                                                      
*             Plate center right ascension in radians                         
*       plt_center_dec                                                        
*          double, Value                                                      
*             Plate center declination in radians                             
*      xi                                                                     
*          double, Value                                                      
*             Input xi standard coordinate in arcseconds.                     
*      eta                                                                    
*          double, Value                                                      
*             Input eta standard coordinate in arcseconds.                    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       ra                                                                    
*          double, Reference                                                  
*             Resulting right ascension in radians.                           
*       dec                                                                   
*          double, Reference                                                  
*             Resulting declination in radians.                               
*                                                                             
* History:                                                                    
*    Created    : 07-DEC-90 by R. White,                                      
*       Converted to Fortran from IDL routine.                                
*    Delivered  : 31-JUL-91 by R. White,                                      
*       Convert from Fortran to C.                                            
*                                                                             
*    Updated    : 31-JAN-94 by J. Doggett,                                    
*       Adopted into software used to facilitate the accessing of data from   
*          the CD-ROM set of the Digitized Sky Survey.                        
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
extern void
transtdeq(plt_center_ra, plt_center_dec,xi,eta,ra,dec)
double plt_center_ra, plt_center_dec;
double xi, eta;
double *ra, *dec;
{
double object_xi,object_eta,numerator,denominator;

    /*
     *  Convert to radians
     */
    object_xi = xi/ARCSECONDS_PER_RADIAN;
    object_eta = eta/ARCSECONDS_PER_RADIAN;

/* *** transtdeq ***                                                       2 */


    /*
     *  Convert to RA and Dec
     */
    numerator = object_xi/cos(plt_center_dec);
    denominator = 1-object_eta*tan(plt_center_dec);
    *ra = atan2(numerator,denominator)+plt_center_ra;
    if (*ra < 0.0) *ra = (*ra)+2*PI;

    numerator = cos((*ra) - plt_center_ra);
    denominator = (1-object_eta*tan(plt_center_dec))/
                (object_eta+tan(plt_center_dec));
    *dec = atan(numerator/denominator);
}
/******************************************************************************
* Module:                                                                     
*    Write_DMS                                                                
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    This routines takes equatorial coordinate in radians and writes them to  
*    a file in hours, minutes, seconds, degrees, arcminutes, arcseconds.      
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Output_File                                                           
*          File*, Value                                                       
*             Pointer to file where coordinates are to be written.            
*       RA                                                                    
*          double, Value                                                      
*             Right ascension to write.                                       
*       Dec                                                                   
*          double, Value                                                      
*             Declination to write.                                           
*       Format                                                                
*          STRING, Reference                                                  
*             Format string for writing coordinates.  The resulting output
*             must be 132 characters or less.
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       Adopted from GASP Fortran into C for software used to facilitate the  
*          accessing of data from the CD-ROM set of the Digitized Sky Survey. 
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 26-MAY-94 by J. Doggett,                                    
*       Some operating systems such as Ultrix have a none ANSI standard       
*       fprintf function.  They return zero for success rather than the       
*       number of characters written.  Write_DMS trapped non-positive         
*       return values as an error.  Since none of the other calls to          
*       fprintf in GetImage are checked for status, we'll just eliminate it   
*       here.                                                                 
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
* 
*    Updated    : 19-DEC-96 by J. Doggett,
*       Modified to not output 60 in the RA_Minutes, RA_Seconds,
*       Dec_ArcMinutes or Dec_ArcSeconds fields and not 24 in RA_Hours field.
*    Redelivered: 19-DEC-96 by J. Doggett,
*       GetImage Version 2.0x.
******************************************************************************/
/*************************
* Write_DMS declarations *
*************************/
void Write_DMS( Output_File, RA, Dec, Format )
FILE* Output_File;
double RA        ;

/* *** Write_DMS ***                                                       2 */


double Dec       ;
STRING Format    ;
{
   int RA_Hours         ; /* Place to hold reformatted coordinates while */
   int RA_Minutes       ; /* they're written out.                        */
   double RA_Seconds    ;
   char   Dec_Sign      ;
   int    Dec_Degrees   ;
   int    Dec_ArcMinutes;
   double Dec_ArcSeconds;

   int N_Chars_Written;

   char   test_string[133];

/**********************************
* Write_DMS begin executable code *
**********************************/
   conrddms( RA, Dec,
         &RA_Hours              , &RA_Minutes    , &RA_Seconds    ,
         &Dec_Sign, &Dec_Degrees, &Dec_ArcMinutes, &Dec_ArcSeconds  );

   sprintf( test_string, Format,
      RA_Hours             , RA_Minutes    , RA_Seconds    ,
      Dec_Sign, Dec_Degrees, Dec_ArcMinutes, Dec_ArcSeconds  );
   
   sscanf(test_string, Format, 
      &RA_Hours              , &RA_Minutes    , &RA_Seconds    ,
      &Dec_Sign, &Dec_Degrees, &Dec_ArcMinutes, &Dec_ArcSeconds  );

   if (RA_Seconds == 60.)
   {
      RA_Seconds =  0.;
      RA_Minutes += 1 ;
   }       
   if (RA_Minutes == 60)
   {
      RA_Minutes =  0;
      RA_Hours   += 1;
   }
   if (RA_Hours == 24)
      RA_Hours = 0;

   if (Dec_ArcSeconds == 60.)
   {
      Dec_ArcSeconds =  0.;
      Dec_ArcMinutes += 1 ;
   }
   if (Dec_ArcMinutes == 60)
   {
      Dec_ArcMinutes =  0;
      Dec_Degrees    += 1;
   }

   N_Chars_Written =
      fprintf( Output_File, Format,
         RA_Hours             , RA_Minutes    , RA_Seconds    ,
         Dec_Sign, Dec_Degrees, Dec_ArcMinutes, Dec_ArcSeconds  );

   return;
}

/* *** Write_DMS ***                                                       3 */
   

/********************************
* Write_DMS end executable code *
********************************/

/******************************************************************************
* Module:                                                                     
*    traneqstdEQ                                                                
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*    Routine to convert RA and Dec in radians to standard coordinates on a    
*    plate using Equidistant projection.                                                                   
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       plt_center_ra                                                         
*          double, Value                                                      
*             Plate center right ascension in radians                         
*       plt_center_dec                                                        
*          double, Value                                                      
*             Plate center declination in radians                             
*       object_ra                                                             
*          double, Value                                                      
*             Input right ascension in radians.                               
*       object_dec                                                            
*          double, Value                                                      
*             Input declination in radians.                                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       Object_Xi                                                             
*          double, Reference                                                  
*             Output xi standard coordinate in arcseconds.                    
*       Object_Eta                                                            
*          double, Reference                                                  
*             Output eta standard coordinate in arcseconds.                   
*                                                                             
* History:                                                                    
*    Created    : 01-AUG-2005 by B.McLean,                                                                  
*       GetImage Version 3.0.                                                 
******************************************************************************/
extern void
traneqstdEQ(plt_center_ra, plt_center_dec,object_ra,object_dec,object_xi,object_eta)
double plt_center_ra, plt_center_dec;
double object_ra, object_dec;
double *object_xi, *object_eta;
{
double div, theta, stheta,factor;

    /* Find factor*/
	stheta=(sin(object_dec)*sin(plt_center_dec)+cos(object_dec)*cos(plt_center_dec)*cos(object_ra-plt_center_ra));
	theta=acos(stheta);
	factor=120.0/(120.0-20.0*theta*theta+theta*theta*theta*theta);
	
	/* Compute standard coords and convert to arcsec*/
    *object_xi=cos(object_dec)*sin(object_ra - plt_center_ra)*
        ARCSECONDS_PER_RADIAN*factor;

    *object_eta=(sin(object_dec)*cos(plt_center_dec)-
        cos(object_dec)*sin(plt_center_dec)*
        cos(object_ra - plt_center_ra))*
        ARCSECONDS_PER_RADIAN*factor;

}



/******************************************************************************
*    transtdeqEQ                                                                
* Module:                                                                     
*                                                                             
* File:                                                                       
*    astronmy.c                                                               
*                                                                             
* Description:                                                                
*     Routine to convert standard coordinates on a plate to RA and Dec in     
*     radians using an Equidistant projection.                                                                
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       plt_center_ra                                                         
*          double, Value                                                      
*             Plate center right ascension in radians                         
*       plt_center_dec                                                        
*          double, Value                                                      
*             Plate center declination in radians                             
*      xi                                                                     
*          double, Value                                                      
*             Input xi standard coordinate in arcseconds.                     
*      eta                                                                    
*          double, Value                                                      
*             Input eta standard coordinate in arcseconds.                    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       ra                                                                    
*          double, Reference                                                  
*             Resulting right ascension in radians.                           
*       dec                                                                   
*          double, Reference                                                  
*             Resulting declination in radians.                               
*                                                                             
* History:                                                                    
*    Created    : 01-AUG-2005 by B.McLean,                                                           
*       GetImage Version 3.0.                                                 
******************************************************************************/
extern void
transtdeqEQ(plt_center_ra, plt_center_dec,xi,eta,ra,dec)
double plt_center_ra, plt_center_dec;
double xi, eta;
double *ra, *dec;
{
double object_xi,object_eta,numerator,denominator;
double theta, theta_factor, ra_int;

    /*
     *  Convert to radians
     */
    object_xi = xi/ARCSECONDS_PER_RADIAN;
    object_eta = eta/ARCSECONDS_PER_RADIAN;

    /*
     *  Convert to RA and Dec
     */
	 
	theta=sqrt(object_xi*object_xi+object_eta*object_eta);
	theta_factor=(120.0-20.0*theta*theta+theta*theta*theta*theta)/120.0;
	
	*dec=asin(object_eta*cos(plt_center_dec)*theta_factor+sin(plt_center_dec)*cos(theta));
	
	ra_int=object_xi/cos(*dec)*theta_factor;
	if (ra_int < -1) ra_int=-1;
	if (ra_int > +1) ra_int=+1;
	*ra=asin(ra_int)+plt_center_ra;
	 
    }

/******************************************************************************
*                                                                   
* Module: Refraction_Correction                                                                                                                 
*                                                                             
* Description:                                                                
*     Routine to adjust measured x,y for refraction                                                               
*                                                                             
*    Input/Output Arguments:                                                  
*       object_x, object_y : Measured coordinates in mm
*                                                                             
* History:                                                                    
*    Created    : 01-AUG-2005 by B.McLean,                                                           
*       GetImage Version 3.0.                                                 
******************************************************************************/
extern void
Refraction_Correction(measured_x,measured_y,Scale,ref_Beta,ref_BetaP,ref_k1,ref_k2,corrected_x,corrected_y)
double measured_x, measured_y, *corrected_x, *corrected_y;
double Scale, ref_Beta, ref_BetaP, ref_k1, ref_k2;
{
double Beta, BetaP, k1, k2, k3, k4;
double x, y, xc, yc, xc1, xc2, yc1, yc2;

    /* Convert to radians */
    Beta = ref_Beta/ARCSECONDS_PER_RADIAN;
    BetaP = ref_BetaP/ARCSECONDS_PER_RADIAN;
	k1 = ref_k1/ARCSECONDS_PER_RADIAN;
	k2 = ref_k2/ARCSECONDS_PER_RADIAN;
	k3 = 1.0+k1*k1;
	k4 = 1.0+k2*k2;
	x = measured_x*Scale/ARCSECONDS_PER_RADIAN;
	y = measured_y*Scale/ARCSECONDS_PER_RADIAN;

    /* Compute corrections */
	xc1=2.0*(Beta+2.0*BetaP*(1.0+k1*k1+k2*k2))*k1*k2*y;
	xc2=-Beta*k1*k3*x*x
	    -2.0*Beta*k1*k1*k2*x*y
		-Beta*k1*k4*y*y
		+Beta*k3*k3*x*x*x
		+3.0*Beta*k1*k2*k3*x*x*y 
		+Beta*(k3*k4+2.0*k1*k1*k2*k2)*x*y*y
		+Beta*k1*k2*k4*y*y*y;
	xc=xc1+xc2;
		
	yc1=(Beta+2.0*BetaP*(1.0+k1*k1+k2*k2))*(k2*k2-k1*k1)*y;
	yc2=-Beta*k2*k3*x*x
		-2.0*Beta*k1*k2*k2*x*y
		-Beta*k2*k4*y*y
		+Beta*k1*k2*k3*x*x*x
		+Beta*(k3*k4+2.0*k1*k1*k2*k2)*x*x*y
		+3.0*Beta*k1*k2*k4*x*y*y
		+Beta*k4*k4*y*y*y;
	yc=yc1+yc2;
	/*printf("%f %f %f %f %f %f %f %f %f %f %f\n",Scale,Beta,BetaP,k1,k2,k3,k4,x,y,xc,yc);*/
		
	/* Apply correction */
	*corrected_x=(x+xc)/Scale*ARCSECONDS_PER_RADIAN;
	*corrected_y=(y+yc)/Scale*ARCSECONDS_PER_RADIAN;
    }
	

