/******************************************************************************
* C Source: Extraction                                                        
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    This set of routines manipulate a structure (see extrctn.h) which        
*    contains data about a plate image data extraction, namely, the Field     
*    (see field.c and field.h) for the extraction, and the image section (see 
*    imgsectn.c and imgsectn.h) to be extracted.                              
*                                                                             
* Module List:                                                                
*    Compare_Extractions                                                      
*       Lexigraphically compares data items of two extractions.               
*    Delete_Extraction                                                        
*       Free memory for an extraction.                                        
*    Get_Extraction                                                           
*       Gets image section for extraction of a field from a plate.            
*    Read_Extraction_Info                                                     
*       Read data for a single extraction from a file.                        
*    Write_Extraction_Information                                             
*       Write extraction information to a file.                               
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 20-JUL-94 by J. Doggett,                                    
*       Updated Read_Extraction_Info function.                                
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 14-FEB-95 by J. Doggett,                                    
*       Updated Read_Extraction_Info function.                                
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Updated Read_Extraction_Info and Write_Extraction_Information         
*       functions.                                                            
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 12-SEP-96 by J. Doggett,                                    
*       Redelivered  Write_Extraction_Information function.	       
*    Updated    : 24-OCT-96 by J. Doggett,                                    
*       Redelivered Compare_Extractions functions.                     
*    Updated    : 28-OCT-96 by J. Doggett,                                    
*       Redelivered Read_Extraction_Info function.                           
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993-1996, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

/* *** Extraction Introduction ***                                        2 */

#include <string.h>

#include "extrctn.h"

#include "astronmy.h"
#include "field.h"
#include "fieldlst.h"
#include "generic.h"
#include "hdrlist.h"
#include "header.h"
#include "imgsectn.h"
#include "imgsnlst.h"
#include "plate.h"
#include "platelst.h"

/******************************************************************************
* Module:                                                                     
*    Compare_Extractions                                                      
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    Makes a comparison of two extractions based, in order, on the field      
*    name, plate id of the image section, and all other data items of the     
*    field.                                                                   
*                                                                             
* Return Value:                                                               
*    int: The result of the comparison of the first data item that is         
*       different. If all the items are equal, then zero is returned.         
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Extraction_1                                                          
*          EXTRACTION, Reference                                              
*             The first extraction to compare.                                
*       EXTRACTION_2                                                          
*          EXTRACTION, Reference                                              
*             The second extraction to compare.                               
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 24-OCT-96 by J. Doggett,                                    
*       Modified to compare plates using the Compare_Plates function.         
*    redelivered: 24-OCT-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***********************************
* Compare_Extractions declarations *
***********************************/
int        Compare_Extractions( Extraction_1, Extraction_2 )
EXTRACTION Extraction_1;
EXTRACTION Extraction_2;
{
   int Comparison; /* Result of the comparison to return. */

/********************************************
* Compare_Extractions begin executable code *
********************************************/
   Comparison = strcmp( Extraction_1->Field->Name, Extraction_2->Field->Name );
   if ( Comparison != 0 )
      return Comparison;

   Comparison = Compare_Plates( Extraction_1->Plate, Extraction_2->Plate );
   if ( Comparison != 0 )
      return Comparison;


/* *** Compare_Extractions ***                                            2 */


   Comparison = Compare_Fields( Extraction_1->Field, Extraction_2->Field );
   if ( Comparison != 0 )
      return Comparison;

   return 0;
}
/******************************************
* Compare_Extractions end executable code *
******************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Extraction                                                        
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    Frees memory previously allocated for an extraction, but doesn't delete  
*    the field or image section structures.                                   
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Extraction                                                            
*          EXTRACTION, Reference                                              
*             The extraction we're deleting from memory.                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************
/*********************************					      
* Delete_Extraction declarations *					      
*********************************/					      
void       Delete_Extraction( Extraction )				      
EXTRACTION Extraction;							      
{									      
/******************************************				      
* Delete_Extraction begin executable code *				      
******************************************/				      
   if ( Extraction != NULL )						      
      free( Extraction );						      
									      
   return;								      
}									      
/****************************************				      
* Delete_Extraction end executable code *				      
****************************************/				      
									      
/*****************************************************************************
* Module:                                                                     
*    Get_Extraction                                                           
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    Puts together data for an extraction of image data from a plate.         
*                                                                             
* Return Value:                                                               
*    EXTRACTION: The resulting extractin.                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Field                                                                 
*          FIELD, Reference                                                   
*             The field for which we want an image section.                   
*       Plate                                                                 
*          PLATE, Reference                                                   
*             The plate from which we're getting an image section.            
*       Image_Section                                                         
*          IMAGE_SECTION, Reference                                           
*             The image section which will be extracted for the plate.        
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/******************************
* Get_Extraction declarations *
******************************/
EXTRACTION    Get_Extraction( Field, Plate, Image_Section )
FIELD         Field           ;
PLATE         Plate           ;
IMAGE_SECTION Image_Section ;
{
   EXTRACTION    Extraction    ; /* New extraction */

/***************************************
* Get_Extraction begin executable code *
***************************************/
   if ( Image_Section == NULL )
      Extraction = NULL;
   else
   {
      Extraction = ( EXTRACTION )Allocate_Memory( sizeof( *Extraction ) );
      Extraction->Field         = Field        ;
      Extraction->Plate         = Plate        ;
      Extraction->Image_Section = Image_Section;
   }

   return Extraction;
 }

/* *** Get_Extraction ***                                                 2 */


/*************************************
* Get_Extraction end executable code *
*************************************/

/******************************************************************************
* Module:                                                                     
*    Read_Extraction_Info                                                     
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    Read data for extraction from a file.                                    
*                                                                             
* Return Value:                                                               
*    EXTRACTION: The extraction read from the file.  If the x-y read from the 
*       file does not equal that computed from the right ascension and        
*       declination, then NULL is returned.                                   
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       InputFile                                                             
*          FILE*, Value                                                       
*             File pointer to the input file.                                 
*       Field_List                                                            
*          FIELD_LIST, Reference                                              
*             List where we'll store the field info constructed from the      
*             extraction.                                                     
*       Image_Section_List                                                    
*          IS_LIST, Reference                                                 
*             List where we'll store info about the image sections for the    
*             extraction.                                                     
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List where we'll store the headers for the plates from which    
*             we're extracting.                                               
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 25-OCT-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 20-JUL-94 by J. Doggett,                                    
*       Modified to reflect change in argument list of gethdr function.       
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 14-FEB-95 by J. Doggett,                                    
*       Relaxed tolerance of input (x,y) versus computed (x,y) to 1 pixel.    
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Modified to reflect increase in limit of field name size.             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 12-SEP-96 by J. Doggett,                                    
******************************************************************************/

/* *** Get_Extraction ***                                                 2 */


/******************************************************************************
*       Modified to read disc volume label rather than disc number to be      
*       consistent with write function.                                       
*    Updated    : 28-OCT-96 by J. Doggett,                                    
*       Modified to reflect change in calling sequence to find_plate_in_list. 
*    redelivered: 28-OCT-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/************************************
* Read_Extraction_Info declarations *
************************************/
EXTRACTION  Read_Extraction_Info
      ( Input_File        ,
        Plate_List        ,
        Header_Directory  ,
        Header_Extension  ,
        Field_List        ,
        Image_Section_List,
        Header_List
      )
FILE       *Input_File        ;
PLATE_LIST  Plate_List        ;
STRING      Header_Directory  ;
STRING      Header_Extension  ;
FIELD_LIST  Field_List        ;
IS_LIST     Image_Section_List;
HDR_LIST    Header_List       ;
{
/* Extraction data read from file */
   char   Field_Name[FIELD_NAME_MAX_LENGTH + 1] ;
   char   Region    [ 7 ] ;
   char   Plate_Id  [ 5 ] ;
   double RA_Radians      ;
   double Dec_Radians     ;
   double Epoch           ;
   int    X               ;
   int    Y               ;
   int    Width           ;
   int    Height          ;
   char   volumeLabel[81] ;
/* Data structures for storing extraction data */
   double Field_Width     ;
   double Field_Height    ;
   FIELD  Field           ;
   IMAGE_SECTION
          Image_Section   ;
   PLATE  Plate           ;
/* Header file stuff */
   HEADER Header          ;
   STRING Header_File_Name;

/* Other stuff */
   char Comment_Character = '#';
   int  Temp_Char;

/*********************************************
* Read_Extraction_Info begin executable code *
*********************************************/
/* Read field name */
   Field_Name[0] = Comment_Character;
   while ( Field_Name[0] == Comment_Character )

/* *** Read_Extraction_Info ***                                           3 */


   {
      Field_Name[0] = getc(Input_File);
      if (Field_Name[0] == Comment_Character)
         while (( (Temp_Char=getc(Input_File)) != '\n')
                                   &&
                (Temp_Char                     != EOF )
               ) ;
      else
      {
         ungetc(Field_Name[0], Input_File);
         switch( fscanf( Input_File, "%s", Field_Name ) )
         {
            case 1:
               break;
            case EOF:
               return ( EXTRACTION )EOF;
            default:
               fprintf(stderr,"Error reading extraction field name.\n");
               return ( EXTRACTION )NULL;
         }
      }
   }
   Field_Name[FIELD_NAME_MAX_LENGTH] = (char)NULL;

   switch( Read_DMS( Input_File, &RA_Radians, &Dec_Radians ) )
   {
/* Read field position */
      case 1:
         break;
      case EOF:
         return ( EXTRACTION )EOF;
      default:
         fprintf(stderr,"Error with input extraction coordinates.\n");
         return ( EXTRACTION )NULL;
   }

/* Read rest of info */
   switch( fscanf( Input_File                  ,
                   "%s %s %f %d %d %d %d %s",
             Region, Plate_Id, &Epoch, &X, &Y, &Width, &Height, volumeLabel ) )
   {
      case 8:
         break;
      case EOF:
         return ( EXTRACTION )EOF;
      default:
         fprintf(stderr,"Error reading extraction information.\n");
         return ( EXTRACTION )NULL;
   }

/* Get header file info */
   Header = Get_Header_from_List( Header_List, Plate_Id );
   if ( Header == NULL )
   {
      Header_File_Name =
            Make_Header_File_Name(Header_Directory, Region, Header_Extension);
      Header = gethdrext(Header_File_Name);
      free(Header_File_Name);
      if ( Header == NULL )
         return ( EXTRACTION )NULL;


/* *** Read_Extraction_Info ***                                           4 */


      Add_Header_to_List( Header_List, Header );
   }

/* Build data structures */
   Field_Width = Width                /* From pixels   */
                *Header->x_pixel_size /* To microns    */
                /1000.                /* To mm         */
                *Header->plt_scale    /* To arcseconds */
                /60.;                 /* To arcminutes */
   Field_Height = Height               /* From pixels   */
                 *Header->y_pixel_size /* To microns    */
                 /1000.                /* To mm         */
                 *Header->plt_scale    /* To arcseconds */
                 /60.;                 /* To arcminutes */

   Field = Create_Field( Field_Name, RA_Radians, Dec_Radians, Width, Height );
   if ( Field == NULL )
      return ( EXTRACTION )NULL;
   Plate = Find_Plate_in_List( Plate_List, Plate_Id, volumeLabel );
      if ( Plate == NULL )
         return ( EXTRACTION )NULL;

   Image_Section = Get_Image_Section( Field, Plate, Header );

      if ( Image_Section == NULL )
         return ( EXTRACTION )NULL;

/* Make sure original x, y, width, height preserved after creating new
 * image section.
 */
   if ((abs(Image_Section->X - X) <= 1)
                &&
       (abs(Image_Section->Y - Y) <= 1)
      ) {
      Image_Section->X      = X;
      Image_Section->Y      = Y;
      Image_Section->Width  = Width;
      Image_Section->Height = Height;

      Add_Field_to_List( Field_List, Field );
      Add_Image_Section_to_List( Image_Section_List, Image_Section );

      return Get_Extraction( Field, Plate, Image_Section );
   }
   else {
      fprintf( stdout,
                "Input x-y inconsistent with input celestial coordinates.\n" );
      Delete_Field        ( Field         );
      Delete_Image_Section( Image_Section );
      return ( EXTRACTION )NULL;
   }
}
/*******************************************
* Read_Extraction_Info end executable code *
*******************************************/

/******************************************************************************
* Module:                                                                     
*    Write_Extraction_Information                                             
*                                                                             
* File:                                                                       
*    extrctn.c                                                                
*                                                                             
* Description:                                                                
*    Write information for an extraction to a file.                           
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Extraction                                                            
*          EXTRACTION, Reference                                              
*             The extraction data to write.                                   
*       File                                                                  
*          FILE*, Value                                                       
*             Pointer to the file to write to.                                
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Modified to reflect increase in limit of field name size.             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 12-SEP-96 by J. Doggett,                                    
*       Modified to write disc volume label rather than disc number.          
*    Redelivered: 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/********************************************
* Write_Extraction_Information declarations *
********************************************/
void        Write_Extraction_Information( Extraction, File )
EXTRACTION  Extraction;
FILE       *File      ;
{
/*****************************************************
* Write_Extraction_Information begin executable code *
*****************************************************/
   fprintf( File, "%-*s", FIELD_NAME_MAX_LENGTH, Extraction->Field->Name );

   Write_DMS
         (
            File, Extraction->Field->Center_RA, Extraction->Field->Center_Dec,
           " %02d %02d %06.3lf %1c%02d %02d %05.2lf"
         );
  fprintf(  File,

/* *** Write_Extraction_Information ***                                   2 */


            " %-6s %-4s %8.3f %5d %5d %5d %5d %s\n",
             Extraction->Plate->Region        ,
             Extraction->Plate->Id            ,
             Extraction->Plate->Epoch         ,
             Extraction->Image_Section->X     ,
             Extraction->Image_Section->Y     ,
             Extraction->Image_Section->Width ,
             Extraction->Image_Section->Height,
             Extraction->Plate->volumeLabel   
          );

   return;
}
/***************************************************
* Write_Extraction_Information end executable code *
***************************************************/
