/******************************************************************************
* C Source: Image                                                             
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    This set of functions processes plate data.                              
*                                                                             
* Module List:                                                                
*    Construct_Image_Header                                                   
*       Construct new header for extracte image.                             
*    Decompress_Image                                                         
*       Read compressed data from tile files, and uncompress into the         
*       desired extracted image.                                              
*    Extract_Image                                                            
*       Extracts image data from compressed tile files.                       
*    FITS_File_Exists                                                         
*       Returns TRUE if the fits file with the input root name exits.         
*    Get_Image_Range                                                          
*       Compute the minimum and maximum data values of an image.              
*    Get_Tile_Size                                                            
*       Returns the width and height of a compressed image tile.              
*    HHD_File_Exists                                                          
*       Returns TRUE if the hhd file with the input root name exits.          
*    HHH_File_Exists                                                          
*       Returns TRUE if the hhh file with the input root name exits.          
*    Image_File_Exists                                                        
*       Returns TRUE is image file(s) exists.                                 
*    Make_Plate_File_Name_Template                                            
*       Construct a template for the names of the compressed tile files for   
*       a plate.                                                              
*    printBriefCopyRight                                                      
*       Prints a copyright statement from a plate header to a file.           
*    Set_Tile_File_Name_Extension                                             
*       Set the extension for a compress tile file name.                      
*                                                                             
* History:                                                                    
*    Created    : 13-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 26-MAY-94 by J. Doggett,                                    
*       Updated Set_Tile_File_Name_Extension function.                        
*    Updated    : 26-OCT-94 by J. Doggett,                                    
*       Updated Decompress_Image function.                                    
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 28-Feb-95 by J. Doggett,                                    
*       Added Get_Tile_Size function.                                         
*       Added math.h include file to use ceil function in Decompress_Image.   
*    Updated    : 30-MAR-95 by J. Doggett,                                    
*       Updated Decompress_Image function.                                    
*    Updated    : 07-APR-95 by J. Doggett,                                    
*       Updated Extract_Image function.                                       
*    Updated    : 30-APR-95 by J. Doggett,                                    
*       Added printBriefCopyRight function.                                   
*       Added decode.h include file for decode prototypes used by             
*       printBriefCopyRight function.                                         
*******************************************************************************

/* *** Image Introduction ***                                              2 */


/******************************************************************************
*    Updated    : 13-JUL-95 by J. Doggett,                                    
*       Updated Extract_Image function.                                       
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 24-SEP-96 by J. Doggett,                                    
*        Redelivered Decompress_Image function.  
*    Updated    : 24-NOV-96 by J. Doggett,                                    
*        Redelivered Extract_Image function.				      
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993-1996, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* access function not prototyped in standard place */

#if defined __VMS
#  include <unixio.h>
#elif defined __UNIX
#  include <unixstd.h>
#else
#endif

#include "image.h"

#include "astrmcal.h"
#include "decode.h"
#include "extrctn.h"
#include "filenams.h"
#include "generic.h"
#include "header.h"
#include "qfile.h"
#include "writeimg.h"

extern void hdecompress();

/******************************************************************************
* Module:                                                                     
*    Construct_Image_Header                                                   
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Construct new image header for an extracted image from the header for    
*    the plate.                                                               
*                                                                             
* Return Value:                                                               
*    HEADER: The new image header.                                            
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Plate_Header                                                          
*          HEADER, Reference                                                  
*             Header for the plate from which the image was extracted.        
*       Extraction                                                            
*          EXTRACTION, Reference                                              
*             Information of about the image extracted.                       
*       Image                                                                 
*          IMAGE_DATA, Reference                                              
*             The image extracted.                                            
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 26-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/**************************************
* Construct_Image_Header declarations *
**************************************/
HEADER     Construct_Image_Header( Plate_Header, Extraction, Image )
HEADER     Plate_Header;
EXTRACTION Extraction;
IMAGE_DATA Image;
{
   HEADER      Image_Header ; /* Header we're constructing  */
   STRING      Object       ; /* Name of object             */
   int         Width        ; /* Size of the field          */
   int         Height       ; /*                            */
   IMAGE_VALUE Data_Min     ; /* Data range of image        */
   IMAGE_VALUE Data_Max     ; /*                            */
   int         Center_X     ; /* Postion on plate of image  */
   int         Center_Y     ; /*    center.                 */
   int         Corner_X     ; /* Position on plate of lower */
   int         Corner_Y     ; /*    left corner of image.   */
   int         RA_Hours     ; /* Center coordinates         */
   int         RA_Minutes   ; /*                            */
   double      RA_Seconds   ; /*                            */
   char        Dec_Sign     ; /*                            */
   int    Dec_Degrees       ; /*                            */
   int    Dec_ArcMinutes    ; /*                            */

/* *** Construct_Image_Header ***                                          2 */


   double Dec_ArcSeconds    ; /*                            */
   float  Object_Mag   = 0. ; /* Dummy variable for xypos   */
   float  Object_Color = 0. ; /* Dummy variable for xypos   */
   float  Object_X = 0.     ; /* X-Y position of object     */
   float  Object_Y = 0.     ; /*    on the plate            */

/***********************************************
* Construct_Image_Header begin executable code *
***********************************************/
   Image_Header  = ( HEADER )Allocate_Memory( sizeof( *Image_Header ) );
   *Image_Header = *Plate_Header                     ;
   Object        = Extraction->Field->Name          ;
   Width         = Extraction->Image_Section->Width ;
   Height        = Extraction->Image_Section->Height;
   Center_X      = Extraction->Image_Section->X     ;
   Center_Y      = Extraction->Image_Section->Y     ;
   Corner_X      = Center_X - Width/2               ;
   Corner_Y      = Center_Y - Height/2              ;

   conrddms( Extraction->Field->Center_RA, Extraction->Field->Center_Dec,
             &RA_Hours, &RA_Minutes, &RA_Seconds, &Dec_Sign, &Dec_Degrees,
             &Dec_ArcMinutes, &Dec_ArcSeconds );

   xypos( Image_Header                                               ,
          Extraction->Field->Center_RA, Extraction->Field->Center_Dec,
          Object_Mag, Object_Color, &Object_X, &Object_Y
        );

/* Put in image info */
   Get_Image_Range(Image, Width*Height, &Data_Min, &Data_Max);
   fillhdr( Image_Header, Object, Width, Height, Corner_X, Corner_Y,
           Data_Min, Data_Max );

/* Put in object info */
   fillpos( Image_Header                                         ,
            RA_Hours, RA_Minutes, RA_Seconds                     ,
            Dec_Sign, Dec_Degrees, Dec_ArcMinutes, Dec_ArcSeconds,
            ( double )Object_X, ( double )Object_Y
          );

   return Image_Header;
}
/*********************************************
* Construct_Image_Header end executable code *
*********************************************/

/******************************************************************************
* Module:                                                                     
*    Decompress_Image                                                         
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Extract image data from compressed tile files.                           
*                                                                             
*   The plate is assumed to be stored in compressed 500x500 blocks (or        
*   500x499 for the last row).  The required blocks are decompressed one at a 
*   time and stored into image.   If part of the image is off the edge of the 
*   plate, zeros are returned.  A warning is printed if the entire image is   
*   off the plate.                                                            
*                                                                             
* Return Value:                                                               
*    IMAGE_DATA: Pointer to the extracted image data.                         
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Extraction                                                            
*          EXTRACTION, Reference                                              
*             The extraction info for the data we're extracting.              
*       File_Name_Template:                                                   
*          STRING, Reference                                                  
*             Root name for tile files.                                       
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 24-AUG-93 by J. Doggett,                                    
*       Adapting Rick White's original getxy function for decompressing image 
*          tile files.                                                        
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*    Updated    : 26-OCT-94 by J. Doggett,                                    
*       Corrected bug in determining the image tile files to decompress.      
*       ("i2=x2/500; if (i2>27) i1 = 27;" changed to                          
*        "i2=x2/500; if (i2>27) i2 = 27;")                                    
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 30-MAR-95 by J. Doggett,                                    
*       Generalized to read images compressed by any tile size.  Required     
*          change to argument list.                                           
*       Corrected incorrect argument list in call to hdecompress.             
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 24-SEP-96 by J. Doggett,                                    
*       Put argument list in call to hdecompress back to the way it was in    
*       version 1.1 because it was actually correct and the change in         
*       version 1.2 broke the code.                                           
*          change to argument list.                                           
*    Updated    : 31-DEC-97 by J. Doggett,
*       Fixed error in image allocation.  It now allocates sizeof(image*)
*       for each pixel (2 bytes) rather than erroneously allocating
*       sizeof(IMAGE_DATA*) (4 bytes).
*    Redelivered: 31-DEC-97 by J. Doggett,                                    
*       GetImage Version 2.0x.
*
******************************************************************************/
/********************************
* Decompress_Image declarations *
********************************/

/* *** Decompress_Image ***                                                2 */


IMAGE_DATA Decompress_Image
         ( Extraction        ,
           File_Name_Template,
           plateWidthPixels  ,
           plateHeightPixels )
EXTRACTION Extraction        ;
STRING     File_Name_Template;
int        plateWidthPixels  ;
int        plateHeightPixels ;
{
   IMAGE_DATA image;
   static char translate[37] = "0123456789abcdefghijklmnopqrstuvwxyz";
   double xc,yc;
   int nx,ny;
   int i, j, ii, jj, i1, i2, j1, j2, b, x0, y0, x1, x2, y1, y2,
       si1, si2, sii, sj1, sj2, sjj, imi1, imi2, imj1, imj2,
       actualTileWidthPixels, actualTileHeightPixels, ss,
       nominalTileWidthPixels, nominalTileHeightPixels;
   char *filename;
   int nf;
   int *subim;
   int plateWidthTiles, plateHeightTiles; /* Plate width in tile units */

   int minHorizontalTile, maxHorizontalTile;
   int minVerticalTile  , maxVerticalTile  ;

/*****************************************
* Decompress_Image begin executable code *
*****************************************/
    /*
     * calculate corner position
     */

    xc = Extraction->Image_Section->X;
    yc = Extraction->Image_Section->Y;
    nx = Extraction->Image_Section->Width;
    ny = Extraction->Image_Section->Height;
    x0=xc-(nx/2);
    y0=yc-(ny/2);
/*
 * Get memory for image
 */
    image=(IMAGE_DATA)malloc(sizeof(IMAGE_DATA*)*nx*ny);
    if (image == NULL)
       return image;
    /*
     * get plate file template
     */
    filename = File_Name_Template;
    nf = strlen(filename)+1;
    /*
     * Get nominal size for tiles and plate in tiles
     */
    Set_Tile_File_Name_Extension(filename, '0', '0');
    Get_Tile_Size(filename, &nominalTileWidthPixels, &nominalTileHeightPixels);
    plateWidthTiles  =
          (int)ceil((double)plateWidthPixels /nominalTileWidthPixels );
    plateHeightTiles =
          (int)ceil((double)plateHeightPixels/nominalTileHeightPixels);



/* *** Decompress_Image ***                                                3 */


    /*
     * get range of image pixels which we will extract if we're not off edge
     */
    x1 = x0;
    x2 = x0+nx-1;
    y1 = y0;
    y2 = y0+ny-1;

    /*
     * now get actual range of sections (limited to 0 -- plateSizeTiles - 1)
     */
    minHorizontalTile = 0                  ;
    maxHorizontalTile = plateWidthTiles - 1;
    minVerticalTile   = 0                  ;
    maxVerticalTile   = plateHeightTiles - 1;

    i1 = x1/nominalTileWidthPixels;
       if (i1 < minHorizontalTile) i1 = minHorizontalTile;
    i2 = x2/nominalTileWidthPixels;
       if (i2 > maxHorizontalTile) i2 = maxHorizontalTile;

    j1 = y1/nominalTileHeightPixels;
       if (j1 < minVerticalTile) j1 = minVerticalTile;
    j2 = y2/nominalTileHeightPixels;
       if (j2 > maxVerticalTile) j2 = maxVerticalTile;
    /*
     * initialize image to zero
     */
    for (i=0; i<nx*ny; i++) image[i] = 0;
    if ((i2 < i1) || (j2 < j1)) {
        fprintf(stderr,
            "warning: image section lies entirely off edge of plate\n");
        return;
    }
    /*
     * read each section and copy required pixels to image
     */
    b=0;
/*    fprintf(stderr,"decompressing %1d blocks: ",(i2-i1+1)*(j2-j1+1)); */
    for (i = i1; i <= i2; i++) {
        for (j = j1; j <= j2; j++) {
            /*
             * read subimage i,j
             */
            Set_Tile_File_Name_Extension(filename, translate[j], translate[i]);
            /*
             * Decompress image in filename
   * Returns address subim and size actualTileWidthPixels,actualTileHeightPixels
             */
    hdecompress(&subim,&actualTileHeightPixels,&actualTileWidthPixels,filename);

            if ((actualTileWidthPixels  > nominalTileWidthPixels ) ||
                (actualTileWidthPixels  < 0                      ) ||
                (actualTileHeightPixels > nominalTileHeightPixels) ||
                (actualTileHeightPixels < 0                      )) {
                fprintf(stderr, "\nerror: bad image size for %s\n", filename);
                fprintf(stderr, "should be %3d %3d, but is %3d %3d\n",
                    nominalTileHeightPixels,actualTileWidthPixels,
                    actualTileHeightPixels);
                exit(EXIT_FAILURE);
            }

/* *** Decompress_Image ***                                                4 */


            /*
             * get subscript ranges on subimage, image
             */
            si1 = x1-i*nominalTileWidthPixels;
            si2 = x2-i*nominalTileWidthPixels;
            imi1 = max(-si1,0);
            imi2 = x2-x1-max(si2-(actualTileWidthPixels - 1),0);
            si1 = max(si1,0);
            si2 = min(si2,(actualTileWidthPixels - 1));
            sj1 = y1-j*nominalTileHeightPixels;
            sj2 = y2-j*nominalTileHeightPixels;
            imj1 = max(-sj1,0);
            imj2 = y2-y1-max(sj2-(actualTileHeightPixels-1),0);
            sj1 = max(sj1,0);
            sj2 = min(sj2,actualTileHeightPixels-1);
            sjj = sj1;
            for (jj=imj1; jj<=imj2; jj++) {
                sii = si1;
                for (ii=imi1; ii<=imi2; ii++) {
                    /*
                     * force pixel value into range of short integers
                     */
                    ss = subim[sii+actualTileWidthPixels*sjj];
                    if (ss>32767) {
                        ss = 32767;
                    } else if (ss < -32768) {
                        ss = -32768;
                    }
                    image[ii+nx*jj] = ss;
                    sii += 1;
                }
                sjj += 1;
            }
            free((char *) subim);
        }
    }

/*    fprintf(stderr," extracted image %5d x %5d\n",x2-x1+1,y2-y1+1); */

    return image;
}
/***************************************
* Decompress_Image end executable code *
***************************************/

/******************************************************************************
* Module:                                                                    
*    Extract_Image                                                           
*                                                                            
* File:                                                                      
*    image.c                                                                 
*                                                                            
* Description:                                                               
*    Extract image data from compressed tile files.                          
*                                                                            
* Return Value:                                                              
*    none                                                                    
*                                                                            
* Arguments:                                                                 
*                                                                            
*    Input Arguments:                                                        
*       Extraction                                                           
*          EXTRACTION, Reference                                             
*             The extraction info for the data we're extracting.             
*       Plate_Header                                                         
*          HEADER, Reference                                                 
*             Header for the plate from which we're extracting.              
*       Image_Data_Root_Dir                                                  
*          STRING, Reference                                                 
*             Root directory for plate image data.                           
*       imageFileRoot                                                        
*          STRING, Reference                                                 
*             Root for image file to be created.                             
*       Format                                                               
*          IMAGE_FORMAT, Value                                               
*             Format for output image, ie, FITS or HHD (GEIS).               
*       extractCalibrations						     
*          BOOL, Value							     
*             Controls whether or not calibrations are extracted with the    
*             image.							     
*    Input/Output Arguments:                                                 
*       none                                                                 
*    Output Arguments:                                                       
*       none                                                                 
*                                                                            
* History:                                                                   
*    Created    : 20-AUG-93 by J. Doggett,                                   
*       for the Association of Universities for Research in Astronomy, Inc.  
*          at the Space Telescope Science Institute,                         
*       to facilitate the accessing of data from the CD-ROM set of the       
*          Digitized Sky Survey.                                             
*    Delivered  : 01-MAR-94 by J. Doggett,                                   
*       GetImage Version 1.0.                                                
*                                                                            
*    Updated    : 07-APR-95 by J. Doggett,                                   
*       Modified to print copyright statement when extracting an image.      
*    Updated    : 13-JUL-95 by J. Doggett,                                   
*       Modified call to Decompress_Image function to reflect change in      
*       argument list.                                                       
*    Redelivered: 18-AUG-95 by J. Doggett,                                   
*       GetImage Version 1.2.                                                
*									     
*    Updated    : 25-NOV-96 by J. Doggett,                                   
*        Added calibration file name argument.				     
*    Redelivered: 25-NOV-96 by J. Doggett,                                   
*       GetImage Version 2.0x.                                               
******************************************************************************/



/* *** Extract_Image ***                                                   2 */


/*****************************
* Extract_Image declarations *
*****************************/
void         Extract_Image
      ( Extraction, Plate_Header, Image_Data_Root_Dir, imageFileRoot, Format,
		maskFile, calibrationFile  )
EXTRACTION   Extraction         ;
HEADER       Plate_Header       ;
STRING       Image_Data_Root_Dir;
STRING       imageFileRoot      ;
IMAGE_FORMAT Format             ;
STRING		 maskFile;
STRING       calibrationFile    ;
{
   STRING     File_Name_Template ; /* Iimage data tile file name template */
   IMAGE_DATA Image              ; /* Image to return                     */
   HEADER     Image_Header       ;

/**************************************
* Extract_Image begin executable code *
**************************************/
   printf( "Extracting field %s from plate %s(%s)...",
           Extraction->Field->Name, Extraction->Plate->Id,
           Extraction->Plate->Region );
   if (Plate_Header->numBriefCopyRightStatements > 0)
   {
      printf("\n   %s %s\n"                             ,
             "The data currently being extracted",
             "are subject to the following:"   );
      printBriefCopyRight(Plate_Header, stdout);
      printf("...");
   }

   fflush( stdout );
   File_Name_Template = Make_Plate_File_Name_Template
                             ( Image_Data_Root_Dir, Plate_Header->plate_name );

   Image = Decompress_Image
         ( Extraction                    ,
           File_Name_Template            ,
           Plate_Header->section_x_length,
           Plate_Header->section_y_length);

   free( File_Name_Template );

   if ( Image != NULL )
   {
      Image_Header = Construct_Image_Header( Plate_Header, Extraction, Image );

      if ( Format == HHD )
         Write_HHH_Extracted_Image ( Image, Image_Header, imageFileRoot );
      else
	 Write_FITS_Extracted_Image( Image, Image_Header, imageFileRoot,
				       maskFile, calibrationFile);

      free( Image         );
      free( Image_Header  );
   }
   else
      fprintf(stderr, "\n%s%s%s%s%s\n%s\n\n"       ,
              "Couldn't allocate memory for field ",

/* *** Extract_Image ***                                                   2 */


              Extraction->Field->Name              ,
              " on plate "                         ,
              Extraction->Plate->Id                ,
              ";"                                  ,
              "skipping to next extraction..."
             );
   return;
}
/************************************
* Extract_Image end executable code *
************************************/

/******************************************************************************
* Module:                                                                     
*    FITS_File_Exists                                                         
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Returns TRUE if the fits file with the input root name exits.            
*                                                                             
* Return Value:                                                               
*    BOOL: TRUE if the file exists, otherwise FALSE.                          
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       rootName                                                              
*          STRING, Reference                                                  
*             Root name for the fits image file to check.                     
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 16-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/********************************
* FITS_File_Exists declarations *
********************************/
BOOL         FITS_File_Exists( rootName )
STRING       rootName;
{
   int    rootNameLen        ;
   STRING Extension = ".fits";
   int    extensionLen       ;
   STRING fileName           ;
   int    fileNameLen        ;

/*****************************************
* FITS_File_Exists begin executable code *
*****************************************/
   rootNameLen  = strlen(rootName     );
   extensionLen = strlen(Extension    );
   fileNameLen  = rootNameLen + extensionLen;
   fileName     = (STRING)Allocate_Memory(fileNameLen + 1);
   strcpy(fileName, rootName );
   strcat(fileName, Extension);

   if (access(fileName, 0) == 0)
      return TRUE;
   else
      return FALSE;
}
/***************************************
* FITS_File_Exists end executable code *
***************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Image_Range                                                          
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Compute the minimum and maximum data values of an image.                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Image                                                                 
*          IMAGE_DATA, Reference                                              
*             The image data.                                                 
*       nImagePixels                                                          
*          int, Value                                                         
*             Number of pixels in the image.                                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       pData_Min                                                             
*          IMAGE_DATA, Reference                                              
*             Minimum data value in image.                                    
*       pData_Max                                                             
*          IMAGE_DATA, Reference                                              
*             Maxmimum data value in image.                                   
*                                                                             
* History:                                                                    
*    Created    : 14-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************
* Get_Image_Range declarations *
*******************************/
void       Get_Image_Range(Image, nImagePixels, pData_Min, pData_Max )
IMAGE_DATA Image       ;
int        nImagePixels;
IMAGE_DATA pData_Min   ;
IMAGE_DATA pData_Max   ;
{
   int Pixel; /* For loop index */

/****************************************
* Get_Image_Range begin executable code *
****************************************/
   if ((Image == (IMAGE_DATA)NULL) || (nImagePixels <= 0))
   {
      *pData_Min = 0;
      *pData_Max = 0;
      return;
   }

   *pData_Min = Image[ 0 ];
   *pData_Max = Image[ 0 ];

/* *** Get_Image_Range_Image ***                                           2 */


   for (Pixel = 1; Pixel < nImagePixels; Pixel++)

      if (Image[ Pixel ] > *pData_Max)
         *pData_Max = Image[ Pixel ];

      else if (Image[ Pixel ] < *pData_Min)
         *pData_Min = Image[ Pixel ];

   return;
}
/**************************************
* Get_Image_Range end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Tile_Size                                                            
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*       Reads the size of the image tile stored in a compressed data file.    
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*    Input Arguments:                                                         
*       Tile_File_Name                                                        
*          STRING, Reference                                                  
*             The name of the tile file.                                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       pWidth                                                                
*          int, Reference                                                     
*             The width of the tile in pixels.                                
*       pHeight                                                               
*          int, Reference                                                     
*             The height of the tile in pixels.                               
*                                                                             
* History:                                                                    
*    Created    : 28-FEB-95 by J. Doggett,                                    
*       from segments of source in the files hdcmprss.c and decode.c,         
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2                                                  
******************************************************************************/
/*****************************
* Get_Tile_Size declarations *
*****************************/
void     Get_Tile_Size(Tile_File_Name, pWidth, pHeight)
STRING   Tile_File_Name;
int    * pWidth        ;
int    * pHeight       ;
{
   QFILE * Tile_File;
   char    tmagic[2];

/**************************************
* Get_Tile_Size begin executable code *
**************************************/
   Tile_File = qopen(Tile_File_Name, 512, 0);

        /*
         * read magic value in first two bytes
         */
        qread(Tile_File, tmagic, sizeof(tmagic));
/*      if (memcmp(tmagic,code_magic,sizeof(code_magic)) != 0) {*/
        if (!checkCompressedTileFormat(tmagic)) {
                fprintf(stderr, "bad file format\n");
                exit(-1);
        }
        *pWidth  =readint(Tile_File); /* x size of image             */

/* *** Get_Tile_Size ***                                                   2 */


        *pHeight =readint(Tile_File);                   /* y size of image  */

   qclose(Tile_File);
}
/************************************
* Get_Tile_Size end executable code *
************************************/

/******************************************************************************
* Module:                                                                     
*    HHD_File_Exists                                                          
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Returns TRUE if the hhd file with the input root name exits.             
*                                                                             
* Return Value:                                                               
*    BOOL: TRUE if the file exists, otherwise FALSE.                          
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       rootName                                                              
*          STRING, Reference                                                  
*             Root name for the hhd image file to check.                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 16-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************
* HHD_File_Exists declarations *
*******************************/
BOOL         HHD_File_Exists( rootName )
STRING       rootName;
{
   int    rootNameLen       ;
   STRING Extension = ".hhd";
   int    extensionLen      ;
   STRING fileName          ;
   int    fileNameLen       ;

/****************************************
* HHD_File_Exists begin executable code *
****************************************/
   rootNameLen  = strlen(rootName     );
   extensionLen = strlen(Extension    );
   fileNameLen  = rootNameLen + extensionLen;
   fileName     = (STRING)Allocate_Memory(fileNameLen + 1);
   strcpy(fileName, rootName );
   strcat(fileName, Extension);

   if (access(fileName, 0) == 0)
      return TRUE;
   else
      return FALSE;
}
/**************************************
* HHD_File_Exists end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     
*    HHH_File_Exists                                                          
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Returns TRUE if the hhh file with the input root name exits.             
*                                                                             
* Return Value:                                                               
*    BOOL: TRUE if the file exists, otherwise FALSE.                          
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       rootName                                                              
*          STRING, Reference                                                  
*             Root name for the hhh image file to check.                      
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 16-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************
* HHH_File_Exists declarations *
*******************************/
BOOL         HHH_File_Exists( rootName )
STRING       rootName;
{
   int    rootNameLen       ;
   STRING Extension = ".hhh";
   int    extensionLen      ;
   STRING fileName          ;
   int    fileNameLen       ;

/****************************************
* HHH_File_Exists begin executable code *
****************************************/
   rootNameLen  = strlen(rootName     );
   extensionLen = strlen(Extension    );
   fileNameLen  = rootNameLen + extensionLen;
   fileName     = (STRING)Allocate_Memory(fileNameLen + 1);
   strcpy(fileName, rootName );
   strcat(fileName, Extension);

   if (access(fileName, 0) == 0)
      return TRUE;
   else
      return FALSE;
}
/**************************************
* HHH_File_Exists end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     
*    Image_File_Exists                                                        
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Returns TRUE if the file(s) for an image already exist on disk. If       
*    Format is FITS, then the image file is <imageFileRoot>.fits. If Format   
*    is HHD, then image files are <imageFileRoot>.hhh/<imageFileRoot>.hhd.    
*                                                                             
* Return Value:                                                               
*    BOOL: TRUE if the image file(s) exist, otherwise FALSE.                  
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       imageFileRoot                                                         
*          STRING, Reference                                                  
*             Root name for the image file to check.                          
*       Format                                                                
*          IMAGE_FORMAT, Value                                                
*             Format for the image file, ie, FITS or HHD.                     
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 16-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************
* Image_File_Exists declarations *
*********************************/
BOOL         Image_File_Exists( imageFileRoot, Format )
STRING       imageFileRoot;
IMAGE_FORMAT Format       ;
{
/******************************************
* Image_File_Exists begin executable code *
******************************************/
   if (Format == HHD)
      if (HHH_File_Exists(imageFileRoot))
         return TRUE;
      else
         return HHD_File_Exists(imageFileRoot);
   else
      return FITS_File_Exists(imageFileRoot);
}
/****************************************
* Image_File_Exists end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    Make_Plate_File_Name_Template                                            
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*       Construct a template for the names of the compressed tile files for a 
*       plate.                                                                
*                                                                             
* Return Value:                                                               
*    STRING: Pointer to the filename template.                                
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Image_Data_Root_Dir                                                   
*          STRING, Reference                                                  
*             Root directory for plate image data.                            
*       Plate_Name                                                            
*          STRING, Reference                                                  
*             Root name for tile files.                                       
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 23-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************************
* Make_Plate_File_Name_Template declarations *
*********************************************/
STRING Make_Plate_File_Name_Template( Image_Data_Directory, Plate_Name )
STRING Image_Data_Directory;
STRING Plate_Name          ;
{
   STRING Template_File  ; /* Name of file for template: Plate_Name.zz */
   int    Template_Length; /* Length of file template            */
   STRING Template_Dir   ; /* Directory containing tile files    */
   STRING Template       ; /* The file template we're returning. */

/******************************************************
* Make_Plate_File_Name_Template begin executable code *
******************************************************/
   Template_File = ( STRING )Allocate_Memory( strlen( Plate_Name ) + 4 );
   strcpy( Template_File, Plate_Name );
   strcat( Template_File, ".zz" );
   Template_Dir  = ( STRING )Allocate_Memory( strlen( Plate_Name ) + 1 );
   strcpy( Template_Dir, Plate_Name);

   if (!Set_FileNames_Case(Image_Data_Directory, Template_File)) {
      fprintf(stderr,"\nCould not find copyright file in image directory!\n");
      fprintf(stderr, "Aborting!!!\n\n");

      exit(EXIT_FAILURE);
   }

/* *** Make_Plate_File_Name_Template ***                                   2 */


   if (!Set_FileNames_Case(Image_Data_Directory, Template_Dir)) {
      fprintf(stderr,"\nCould not find copyright file in image directory!\n");
      fprintf(stderr, "Aborting!!!\n\n");
      exit(EXIT_FAILURE);
   }

   Template = Build_Full_File_Name( Image_Data_Directory,
                                    Template_Dir        ,
                                    Template_File
                                  );
   free( Template_File );
   free( Template_Dir  );

   Template_File = Set_FileNames_Version(Image_Data_Directory, Template);

   if (Template_File != Template )
      if (Template_File != NULL )
         free(Template);
      else {
         fprintf(stderr,
               "\nCould not find copyright file in image directory!\n");
         fprintf(stderr, "Aborting!!!\n\n");
         exit(EXIT_FAILURE);
      }

   Template = Template_File;

   return Template;
}
/****************************************************
* Make_Plate_File_Name_Template end executable code *
****************************************************/

/******************************************************************************
* Module:                                                                     
*    printBriefCopyRight                                                      
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Prints a copyright statement from a plate header to a file.              
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Plate_Header                                                          
*          HEADER, Reference                                                  
*             Header for the plate from which the image was extracted.        
*       outputFile                                                            
*          FILE, reference                                                    
*             The file to which the copyright statement is to be written.     
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 30-APR-95 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
******************************************************************************/
/***********************************
* printBriefCopyRight declarations *
***********************************/
void     printBriefCopyRight(Plate_Header, outputFile)
HEADER   Plate_Header;
FILE   * outputFile  ;
{
   int i;

/********************************************
* printBriefCopyRight begin executable code *
********************************************/
   for (i=0; i < Plate_Header->numBriefCopyRightStatements; i++)
      fprintf(outputFile, "        %s\n",
              Plate_Header->briefCopyRightStatements[i]);
}
/******************************************
* printBriefCopyRight end executable code *
******************************************/

/******************************************************************************
* Module:                                                                     
*    Set_Tile_File_Name_Extension                                             
*                                                                             
* File:                                                                       
*    image.c                                                                  
*                                                                             
* Description:                                                                
*    Set the extension for a tile file name given the letters for the         
*    extension.                                                               
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*                                                                             
*    Input Arguments:                                                         
*       none                                                                  
*    Input/Output Arguments:                                                  
*       File_Name                                                             
*          STRING, Reference                                                  
*             Tile file name                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 19-NOV-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 26-MAY-94 by J. Doggett,                                    
*       The case for the extension was determined by the case of the          
*       previous extension.  However, if the previous extension was all       
*       digits, then the case information is lost.  The function is           
*       has been modified to determine the case from the first character      
*       from the end of the file name that is either an upper or a lower case 
*       character.                                                            
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
******************************************************************************/
/********************************************
* Set_Tile_File_Name_Extension declarations *
********************************************/
void Set_Tile_File_Name_Extension
      (FileName, ExtensionCharacter1, ExtensionCharacter2)
STRING FileName           ;
char   ExtensionCharacter1;
char   ExtensionCharacter2;
{
   int  FileNameLength;
   int  ExtensionPos  ;
   int  AlphaPos      ;
   BOOL CaseIsUpper   ;

/*****************************************************
* Set_Tile_File_Name_Extension begin executable code *
*****************************************************/
   FileNameLength = strlen(FileName);



/* *** Set_Tile_File_Name_Extension  ***                                   2 */


   for ( ExtensionPos = FileNameLength - 1; ExtensionPos >= 0; ExtensionPos--)
      if (FileName[ExtensionPos] == '.')
         break;

   for (AlphaPos = FileNameLength - 1; AlphaPos >= 0; AlphaPos--)
      if (isupper(FileName[AlphaPos])
                   ||
          islower(FileName[AlphaPos]))
         break;

   if (ExtensionPos >= 0)
      ExtensionPos++;
   else
      ExtensionPos = FileNameLength - 2;

   if (isupper(FileName[AlphaPos])) {
      FileName[ExtensionPos    ] = toupper(ExtensionCharacter1);
      FileName[ExtensionPos + 1] = toupper(ExtensionCharacter2);
   }
   else if (islower(FileName[AlphaPos])) {
      FileName[ExtensionPos    ] = tolower(ExtensionCharacter1);
      FileName[ExtensionPos + 1] = tolower(ExtensionCharacter2);
   }
   else {
      FileName[ExtensionPos    ] = ExtensionCharacter1;
      FileName[ExtensionPos + 1] = ExtensionCharacter2;
   }

   return;
}
/*********************************************
* Set_FileName_Extension end executable code *
*********************************************/
